﻿using Microsoft.Extensions.DependencyInjection;
using System.Net;
using System.Net.Http.Json;
using UserManager.API.IntegrationTests.Helpers;
using UserManager.Application.Features.Users.Requests;
using UserManager.Infrastructure;

namespace UserManager.API.IntegrationTests.Controllers
{
    [TestClass]
    public class UserControllerIntegrationTests
    {
        private HttpClient _client = null!;
        private CustomWebApplicationFactory<Program> _factory = null!;

        [TestInitialize]
        public void Initialize()
        {
            _factory = new CustomWebApplicationFactory<Program>();

            // Create a database with all migrations applied
            using var scope = _factory.Services.CreateScope();
            var services = scope.ServiceProvider;
            var context = services.GetRequiredService<UserManagerContext>();
            context.Database.EnsureCreated();

            _client = _factory.CreateClient();
        }

        [TestCleanup]
        public void Cleanup()
        {
            _client.Dispose();
            _factory.Dispose();
        }

        [TestMethod]
        public async Task CreateUserAsync_Returns400WithValidation_ForInvalidRequest()
        {
            // Arrange
            var request = new CreateUserRequestDto
            {
                FirstName = "",
                LastName = "",
            };

            // Act
            var response = await _client.PostAsJsonAsync("/api/user", request);

            // Assert
            Assert.AreEqual(HttpStatusCode.BadRequest, response.StatusCode);
            Assert.AreEqual("{\"FirstName\":[\"FirstName is required.\"],\"LastName\":[\"LastName is required.\"]}", response.Content.ReadAsStringAsync().Result);
        }

        [TestMethod]
        public async Task CreateUserAsync_Returns200OK_ForValidRequest()
        {
            // Arrange
            var request = new CreateUserRequestDto
            {
                FirstName = "James",
                LastName = "Smith",
            };

            // Act
            var response = await _client.PostAsJsonAsync("/api/user", request);

            // Assert
            Assert.AreEqual(HttpStatusCode.OK, response.StatusCode);

        }
    }
}
